#
# Three PM Report Program
# November 2014
# Copyright 2014-2015 Networked Robotics Corporation
#
# This program is designed to be customized by our customers
#
# This program produces a report in which a single reading is listed at a given time each day
# for an entire month for a single Tempurity System monitored device.

# The program will choose a reading that is the first reading after the given
# time of day.  All readings are at approximately but not exactly the same time-of-day.
#
# Time Zone is an important input to this utility since all Tempurity Data is stored in Universal Time (UTC)
# Time Zone is extracted - but has to apply to the time zone at the time of the report which is complicated
#  by Daylight Savings Time
#
#  The program reads Raw Data, Calibration Factors (corrected data) can not be used
# 
#
# The monitored device # ,the device name, month, and year are input by the user
#
# The file TempurityReport.txt is created with the report data
#
# Data files are expected to be in the directory tree  C:\Windows\System32\Tempurity\Data\ and its subdirectories
# There are different data directories depending on whether the Tempurity Server is running XP, WIN7/WIN8
#  Also the directory is different depending on whether the computer is a 32 bit or 64 bit machine.




import datetime
import time
import select
import struct
import sys
import subprocess
import telnetlib
import os
import zlib






if __name__ == '__main__':

   debug = False

   # California time zone =8  - hours different from UK time 0

   # Timezone very important - must be run from the Timezone with the Tempurity Server
   # Otherwise this parameter should be changed manually
   # Because of Daylight Savings Time - the Timezone is different for different months
   
   timezone = time.altzone
   timezone = timezone/3600      
    
   
   # 15 = 3PM of course
   # Must be in military hours

   #  CHANGE THIS LINE TO THE TIME at which data should be reported for the day
   reporthour = 15
   
   dataunits = "undefined"

   print ("\n")
   print ("Tempurity System Monthly Report \n")
   print (" ")

   devicenumber = input("Enter the three-digit monitored device number (e.g. 044) ")
   devicename = input("Enter the corresponding Tempurity System monitored device name (e.g. Freezer12) ")
   devicemonth = input("Enter the numeric two-digit month ( e.g. 03) ")
   deviceyear = input("Enter the number four-digit year (e.g. 2014) ")
   addsignature = input("Would you like to add a signature line? (Y/N) ")

   runmonth = time.strftime("%m", time.gmtime())
   imonth = int(runmonth)
   idevicemonth = int(devicemonth)
           
   
           # The below deals with Daylight Savings Time
           # if we are running in summer and the report is for summer
           # do nothing
           # if we are running in winter and the report is for winter
           # also do nothing
           
           # if we are running in summer and the report is for winter
           # add one to timezone
           # if we are running in winter and the report is for summer
           # subtract one from timezone
           # Daylight Savings Time shifts are usually in March and November
           
           
   if (((imonth >2) and (imonth <11)) and ((idevicemonth <3) or (idevicemonth>10))):
            timezone = timezone +1

   if ((imonth <3) or (imonth >10)) and ((idevicemonth >2) and (idevicemonth<11)):
            timezone = timezone -1

   if (time.daylight):
            print ("The suggested timezone for the report month selected is", timezone)
            devicezone = input("You can override the default Timezone if you enter it here: ") 

   # Converts Entered timezone to integer     
   if (devicezone == ""):
            cnothing =0    # sorry coding gods
   else:
            idevicezone = int(devicezone)
            timezone = idevicezone
            
   # This program will be partially wrong in months in which there is a Daylight Savings Time Shift
   # This must be accounted for manually       
           
   
   if (devicemonth=="01"):
           charmonth = "January"
           monthlimit=31
   if (devicemonth=="02"):
           charmonth = "February"
           monthlimit=28
   if (devicemonth=="03"):
           charmonth = "March"
           monthlimit=31
   if (devicemonth=="04"):
           charmonth = "April"
           monthlimit =30
   if (devicemonth=="05"):
           charmonth = "May"
           monthlimit=31
   if (devicemonth=="06"):
           charmonth = "June"
           monthlimit=30
   if (devicemonth=="07"):
           charmonth = "July"
           monthlimit=31
   if (devicemonth=="08"):
           charmonth = "August"
           monthlimit = 31
   if (devicemonth=="09"):
           charmonth = "September"
           monthlimit =30
   if (devicemonth=="10"):
           charmonth = "October"
           monthlimit = 31
   if (devicemonth=="11"):
           charmonth = "November"
           monthlimit = 30
   if (devicemonth=="12"):
           charmonth = "December"
           monthlimit = 31



 # Open the report file and write the report header 
   fileHandleW = open('TempurityReport.txt','w')
   fileHandleW.write("\n\n\nTempurity System Monthly Report of Automated Timepoint Values")
   fileHandleW.write("\n")
   fileHandleW.write("\n           for Monitored Device ")
   fileHandleW.write(devicenumber)
   fileHandleW.write("\n\n")
   fileHandleW.write("Device Name: " )
   fileHandleW.write(devicename)
   fileHandleW.write("        " )
   fileHandleW.write("Device Number: " )
   fileHandleW.write(devicenumber)
   fileHandleW.write("\n")
   fileHandleW.write("_____________________________________________________________________")
   fileHandleW.write ("\n\n")
   fileHandleW.write("Beginning Month Year: ")  
   #fileHandleW.write("Month: ")
   fileHandleW.write(charmonth)
   fileHandleW.write("     ")
   #fileHandleW.write("Year ")
   fileHandleW.write(deviceyear)
   fileHandleW.write("\n\n")
   fileHandleW.write("Daily Reading Time:  ")
   fileHandleW.write(str(reporthour))
   fileHandleW.write(":00")
   fileHandleW.write("\n\n")
   fileHandleW.write("Local Timezone (UTC):  GMT -")
   fileHandleW.write(str(timezone))
   fileHandleW.write("\n")
   # uncomment the next line by deleting the number sign if you want the report run date listed
   #fileHandleW.write("Report run on: " + time.asctime(time.localtime()))
   


   gotTodaysData=0
   dayCtr = 0

   # All of the data for a single month is in the same directory
   # Figure out what that directory is
   # First the base directory:


   # when you uncomment these (take out the #) make sure that the indentation aligns with the other statments
   # Choose this for 32 bit XP Tempurity Servers
   #datadirectory1 = "C:\Windows\System32\Tempurity\Data" 

   # The below is for 64 bit XP machines
   #datadirectory1 = "C:\Windows\Syswow64\Tempurity\Data"
   
   # Choose the line below for WIN7 and 8 64 bit Tempurity Servers
   datadirectory1 = "C:" + "\\" + "Program Files (x86)" + "\\" +  "Networked Robotics" + "\\" + "Tempurity Server" + "\\" +"Tempurity" + "\\" + "Data"
   #C:\Program Files (x86)\Networked Robotics\Tempurity Server\Tempurity\Data

   # Choose the below for WIN7 and 8 32 bit Tempurity Servers
   #datadirectory1 = "C:" + "\\" + "Program Files" + "\\" +  "Networked Robotics" + "\\" + "Tempurity Server" + "\\" +"Tempurity
   # C:\Program Files\Networked Robotics\Tempurity Server\Tempurity\Data

   
   datadirectory2 = str(devicenumber) +"\\"+ str(deviceyear) + "\\" + str(devicemonth) + "\\"  
   datadirectory = datadirectory1 + "\\" + datadirectory2

      
   printedheader=0

   # Get 30 days of data starting with the first of the month
   while (dayCtr<monthlimit):

      dayCtr = dayCtr + 1
      # for deviceday = 1 to 30
      if (dayCtr<10):
         deviceday = "0" + str(dayCtr)
      else:
         deviceday = str(dayCtr)

      # Open the Tempurity System data file,  There is one file per monitored device per day
      # first get the file name
      # Filenames are going to be    
      # filename is ddd_yyyy_mm_dd.dat
      # an example filename is 035_2012_10_01.dat
      
      datafilename =str(devicenumber) + "_" + str(deviceyear) +  "_" + str(devicemonth) + "_" + deviceday + ".dat"

      datafilename = datadirectory + datafilename
      
      #print (datafilename)
      
      gotTodaysData=0
      

       # read a record and see if it is after 3PM in local time
       # Convert Universal to local (  (u +(3600*tz))/86400) + 25569
       # if it is,  then record the reading in the report file

         #open the Tempurity data file and search every record until you find
         # one after the target time of day
         
      try:
         fileHandleR = open(datafilename,'r')
         
         for dataline in fileHandleR:
            recctr =0
            # read every line in the data file but if you already have a value don't do anything else
            
            if (gotTodaysData==0):
               #dataline which holds a single line - time and value data
               datatime = dataline [0:10]
               datavalue = dataline [12:18]
               dataunits = dataline [19:26]
               #print (dataline)
               endposition = dataunits.find("+")
               
               if (endposition != -1):
                  dataunits = dataunits [0: endposition-1]
               
               recctr = recctr + 1
               #print ("Madeithere1")

               if (recctr ==1) and (printedheader==0):
                   printedheader=1
                   #print ("Madeithere2")
                   fileHandleW.write("\n\n\n") 
                   fileHandleW.write("     Date   Local Time   ")
                   if (dataunits=="Celsius"):
                           parameter = "Temperature"
                   elif (dataunits=="%RH"):
                           parameter = "Humidity"                     
                   else:
                           parameter = "Value"
                           
                   fileHandleW.write(parameter)
                   fileHandleW.write("(")
                   fileHandleW.write(dataunits)
                   fileHandleW.write (")")
                   fileHandleW.write("\n") 
               

               # Tempurity records time as Universal Time so you need to know what time zone
               # you are in Universal Time is the number of seconds since Jan 1 1970 
               # and is based on GMT time zone
               
               # Get rid of the day (date) and just get seconds in the day 86400 is seconds in a day

               #print (datatime)
               daysecs = int(datatime) - (timezone*3600)
               #print (daysecs)
               daysecs = daysecs % 86400
               #daysecs = (long(datatime) - (timezone*3600.0) ) % 86400.0
               #daysecs is now the local time in seconds after midnight

               #print (daysecs)

                #each file contains data from midnight local time to midnight local time
                # so that means that a file created by a Chicago server starts at
                # 6 am GMT and runs until 6 am GMT the next day
               

               daythreshold = ( reporthour*3600)  
         
               
               if (daysecs>daythreshold):
                   # found the value we want
                   #print ("GOt it")
               # Get the timezone corrected hour and minutes

                   #temp = (daysecs%3600)/60
                   

                   tempminutes = int((daysecs%3600)/60)
                   if (tempminutes < 10):
                      dayminutes = "0" + str(tempminutes)
                   else:
                      dayminutes = str(tempminutes)

                   temphour = int(daysecs/3600)
                   if ((daysecs/3600)<10):           
                      dayhour = "0" + str(temphour)
                   else:
                      dayhour = str(temphour)

                      #dayhour = int(dayhour)
                      #dayminutes = int(dayminutes)
                    
                   
                   
                   fileHandleW.write("\n")
                   fileHandleW.write("     ")
                   fileHandleW.write(deviceday)
                   fileHandleW.write("     ")
                   fileHandleW.write(dayhour)
                   fileHandleW.write(":")
                   fileHandleW.write(dayminutes)
                   fileHandleW.write("          ")
                   fileHandleW.write(datavalue)
                   # if the reading is over a half hour after the target time - print an asterisk
                   # to flag the result
                   if (daysecs>daythreshold + 1800):
                      fileHandleW.write("*")
                   fileHandleW.write("\n")
                   gotTodaysData=1
                   
        
         fileHandleR.close()  
      except:
         fileHandleW.write("\n")
         fileHandleW.write("     ")
         fileHandleW.write(deviceday)
         fileHandleW.write("     ")
         fileHandleW.write("No Data after Target Time ")
         fileHandleW.write("\n")
        
      
      
       #  
       # Open up a notepad window with the report data
    
       # On Win XP 32 bit data directory is C:\Windows\System32\Tempurity\Data\'device id'\'year'\month\

   fileHandleW.write("\n\n")

   
   if ((addsignature == "Y") or (addsignature == "y")) :
      fileHandleW.write("\n\n\n\n")
      fileHandleW.write(" Reviewed by Signature ______________________________________")
      fileHandleW.write("\n\n\n\n")
      fileHandleW.write(" Released by Signature ______________________________________") 
      
   fileHandleW.close()

    # open a notepad window and print to show the results
   #r = os.popen("notepad /p Tempuritymonthreport.txt")
   r = os.popen("notepad  TempurityReport.txt")  
   print ("\n\n")
   sys.exit()

   


